/*
 * IcmpMutex.c
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
//
//
// ICMPMUTEX.C -- Mutrex used by ICMP 
// 
//
#include <windows.h>
#include "Sddl.h"
#include "IcmpDebug.h"
#include "IcmpMutex.h"


/*
 * Create (or open existing) mutex granting rights to everyone
 */
HANDLE IcmpCreateMutex(void)
{
   HANDLE handle; // mutex handle
   SECURITY_ATTRIBUTES sa; // Security attributes structure
   BOOL result;   // operation result
   ULONG dummy = 0; // dummy 

   IcmpOutputDebugString("IcmpCreateMutex IN\n");
   // Check is mutex already exists
   handle = OpenMutex(SYNCHRONIZE, FALSE, ICMP_MUTEX_NAME);
   if (handle == NULL)
   {
      IcmpOutputDebugString("IcmpCreateMutex Needs cretion\n");
      /* Initialize security attributes */
      sa.nLength = sizeof(SECURITY_ATTRIBUTES);
      sa.bInheritHandle = 0;
      sa.lpSecurityDescriptor = 0;
      
      /* Convert security descriptor string to security descriptor (allocated structure must be freed) */
      result = ConvertStringSecurityDescriptorToSecurityDescriptor(ICMP_MUTEX_RIGHTS, SDDL_REVISION_1, &(sa.lpSecurityDescriptor), &dummy);
      if (result)
      {
         /* Create mutex */
         handle = CreateMutex(&sa, FALSE, ICMP_MUTEX_NAME);
         
         /* Free security descriptor */
         LocalFree(sa.lpSecurityDescriptor);
      }
   };
   IcmpOutputDebugString("IcmpCreateMutex OUT: %p\n", handle);
   return handle;   
} /* IcmpCreateMutex */


/*
 * Release mutex (just by closing handle)
 */
BOOL IcmpCloseMutex(HANDLE handle)
{
   BOOL result;
   
   IcmpOutputDebugString("IcmpCloseMutex IN: %p\n", handle);
   result = CloseHandle(handle);
   IcmpOutputDebugString("IcmpCloseMutex OUT: %p %d\n", handle, result);
   return result;
} /* IcmpCloseMutex */


BOOL IcmpLockMutex(HANDLE handle)
{
   BOOL result;
   
   IcmpOutputDebugString("IcmpLockMutex IN: %p\n", handle);
   result = (WaitForSingleObject(handle, INFINITE) == WAIT_OBJECT_0);
   IcmpOutputDebugString("IcmpLockMutex OUT: %p %d\n", handle, result);
   return result;
} /* IcmpLockMutex */


BOOL IcmpUnlockMutex(HANDLE handle)
{
   BOOL result;
   IcmpOutputDebugString("IcmpUnlockMutex IN: %p\n", handle);
   result = ReleaseMutex(handle);
   IcmpOutputDebugString("IcmpUnlockMutex OUT: %p %d\n", handle, result);
   return result;
} /* IcmpUnlockMutex */
